/*
 * Copyright (c) 2010-2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.vew.autoexport.Auftrag
 *
 * de.inovat.vew.autoexport.Auftrag is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.vew.autoexport.Auftrag is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.vew.autoexport.Auftrag; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.vew.autoexport;

//~ JDK IMPORTE ===============================================================

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

//~ KLASSEN ===================================================================

/**
 * Enthlt alle Informationen fr einen Aggregationsauftrag.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision: 780 $ / $Date: 2012-10-31 11:44:41 +0100 (Mi, 31 Okt 2012) $ / ($Author: HCK $)
 */
public class Auftrag {

    /** BASt-Band-Format, welches erzeugt werden soll */
    private BastFormat _bastFormat;

    /** IntervallBeginn */
    private long _intervallBeginn;

    /** IntervallEnde: Intervallbeginn + Intervalllnge (ausschlielich!, also der nchste Intervallbeginn */
    private long _intervallEnde;

    /** VewLZZS - Verwaltungsobjekt, welches den Auftrag ausfhren soll */
    private VewLZZS _vewLZZS;

    /** Zeitpunkt der geplanten Ausfhrung fr diesen Auftrag in Millisekunden seit 1.1.1970 */
    private long _ausfuehrungsZeitPunkt = 0;

    /** Datumsformatierung fr Ausgabe toString() */
    SimpleDateFormat _datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor, um einen Folgeauftrag auf Basis eines bestehenden Auftrags zu berechnen
     *
     * @param auftrag Aktueller Auftrag, fr den der Folgeauftrag bestimmt werden soll
     */
    public Auftrag(Auftrag auftrag) {
        this._vewLZZS    = auftrag.getVewLZZS();
        this._bastFormat = auftrag.getBastFormat();

        long     minimaleWarteZeit = auftrag.getAusfuehrungsZeitPunkt() - auftrag.getIntervallEnde();
        Calendar intervallBeginn   = new GregorianCalendar();

        intervallBeginn.setTimeInMillis(auftrag.getIntervallBeginn());

        Calendar intervallEnde = new GregorianCalendar();

        intervallEnde.setTimeInMillis(auftrag.getIntervallEnde());

        // Unterscheidung AXL (+1 Tag) und LZZS (+1 Monat)
        if (_bastFormat.equals(BastFormat.AXL2009)) {
            intervallBeginn.add(Calendar.DAY_OF_YEAR, 1);
            this._intervallBeginn = intervallBeginn.getTimeInMillis();
            intervallEnde.add(Calendar.DAY_OF_YEAR, 1);
            this._intervallEnde = intervallEnde.getTimeInMillis();
        } else {
            intervallBeginn.add(Calendar.MONTH, 1);
            this._intervallBeginn = intervallBeginn.getTimeInMillis();
            intervallEnde.add(Calendar.MONTH, 1);
            this._intervallEnde = intervallEnde.getTimeInMillis();
        }

        this._ausfuehrungsZeitPunkt = this._intervallEnde + minimaleWarteZeit;
    }

    /**
     * Konstruktor, um den initialen Erstauftrag fr ein Verwaltungsobjekt und das erste zu berechnende Intervall
     * anzulegen
     *
     * @param ausfuehrungsZeitpunkt Zeitpunkt, zu dem Auftrag frhestens ausgefhrt werden soll.
     * @param intervallBeginn       Intervallbeginn (EINSCHLIESSLICH!!) in Millisekunden seit 1.1. 1970
     * @param intervallEnde         Intervallende (AZSSCHLIESSLICH!!) in Millisekunden seit 1.1. 1970
     * @param vewLZZS               Verwaltungsobjekt, welches den Auftrag letztendlich ausfhren soll
     * @param bastFormat            Format des zu erzeugenden BASt-Bandes (V2004, V2007, AXL2009)
     */
    public Auftrag(long ausfuehrungsZeitpunkt, long intervallBeginn, long intervallEnde, VewLZZS vewLZZS, BastFormat bastFormat) {
        _ausfuehrungsZeitPunkt = ausfuehrungsZeitpunkt;
        _intervallBeginn       = intervallBeginn;
        _intervallEnde         = intervallEnde;
        _vewLZZS               = vewLZZS;
        _bastFormat            = bastFormat;
    }

    //~ METHODEN ==============================================================

    /**
     * Fhrt den Auftrag aus, indem am zugeordneten Verwaltungsobjekt die Methode {@link
     * de.inovat.dua.aggrlzzsdtv.VewLZZS#ausfuehren(de.inovat.dua.aggrlzzsdtv.Auftrag)} aufgerufen wir.
     *
     * @return Meldung mit dem Ergebnis der Ausfhrung des Auftrags.
     */
    public String ausfuehren() {
        return _vewLZZS.ausfuehren(this);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert den Zeitpunkt der geplanten Ausfhrung fr diesen Auftrag in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt der geplanten Ausfhrung fr diesen Auftrag in Millisekunden seit 1.1.1970
     */
    public long getAusfuehrungsZeitPunkt() {
        return _ausfuehrungsZeitPunkt;
    }

    /**
     * Liefert das BASt-Band-Format fr diesen Auftrag.
     *
     * @return BASt-Band-Format fr diesen Auftrag.
     */
    public BastFormat getBastFormat() {
        return _bastFormat;
    }

    /**
     * Liefert den Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallBeginn() {
        return _intervallBeginn;
    }

    /**
     * Liefert den Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallEnde() {
        return _intervallEnde;
    }

    /**
     * Liefert das Verwaltungsobjekt, welches den Auftrag ausfhren soll.
     *
     * @return Verwaltungsobjekt, welches den Auftrag ausfhren soll.
     */
    public VewLZZS getVewLZZS() {
        return _vewLZZS;
    }

    //~ METHODEN ==============================================================

    @Override

    /**
     * Gibt die Textreprsentation fr einen Auftrag aus.
     */
    public String toString() {
        String ausfuehrung = "DauerAuftrag";
        String art         = "unbekannt";

        switch (_bastFormat) {
        case V2004 :
            art = "V2004";

            break;

        case V2007 :
            art = "V2007";

            break;

        case AXL2009 :
            art = "AXL2009";

            break;
        }

        return String.format("Auftrag (%-12s- %-15s) Ausfhrung am/um %s: Intervall %s - %s fuer %s", art, ausfuehrung, _datumsFormatierer.format(new Date(_ausfuehrungsZeitPunkt)), _datumsFormatierer.format(new Date(_intervallBeginn)), _datumsFormatierer.format(new Date(_intervallEnde)), _vewLZZS.getPid());
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 31.10.12
